# Copyright 2021-2022 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

import argparse
import subprocess
import os
import ast

cur_dir = os.getcwd()
code_name = 'vae_task'

parser = argparse.ArgumentParser(description="Run {} case".format(code_name))
parser.add_argument("--enable_fl", type=bool, default=True)
parser.add_argument("--device_target", type=str, default="CPU")
parser.add_argument("--server_mode", type=str, default="FEDERATED_LEARNING")
parser.add_argument("--server_num", type=int, default=3)
parser.add_argument("--scheduler_ip", type=str, default="127.0.0.1")
parser.add_argument("--scheduler_port", type=int, default=4444)
parser.add_argument("--scheduler_manage_port", type=int, default=6666)
parser.add_argument("--config_file_path", type=str, default="{}/config.json".format(cur_dir))
parser.add_argument("--enable_ssl", type=ast.literal_eval, default=False)
parser.add_argument("--ms_node_id", type=str, default="fl-scheduler")
# vae args
parser.add_argument("--server_config_path", type=str, default="./Horizontal_FL/conf/server_config.yaml")

args, _ = parser.parse_known_args()
enable_fl = args.enable_fl
device_target = args.device_target
server_mode = args.server_mode
server_num = args.server_num
scheduler_ip = args.scheduler_ip
scheduler_port = args.scheduler_port
scheduler_manage_port = args.scheduler_manage_port
config_file_path = args.config_file_path
enable_ssl = args.enable_ssl
ms_node_id = args.ms_node_id
# vae args
server_config_path = args.server_config_path
os.environ['MS_NODE_ID'] = ms_node_id

cmd_sched = "unset http_proxy && unset https_proxy && "
cmd_sched += "rm -rf {}/scheduler/ && ".format(cur_dir)
cmd_sched += "mkdir {}/scheduler/ && ".format(cur_dir)
cmd_sched += "export GLOG_v=1 && "
cmd_sched += "python {}/{}.py".format(cur_dir, code_name)
cmd_sched += " --enable_fl={}".format(enable_fl)
cmd_sched += " --device_target={}".format(device_target)
cmd_sched += " --server_mode={}".format(server_mode)
cmd_sched += " --ms_role=MS_SCHED"
cmd_sched += " --server_num={}".format(server_num)
cmd_sched += " --config_file_path={}".format(config_file_path)
cmd_sched += " --enable_ssl={}".format(enable_ssl)
cmd_sched += " --scheduler_ip={}".format(scheduler_ip)
cmd_sched += " --scheduler_port={}".format(scheduler_port)
cmd_sched += " --scheduler_manage_port={}".format(scheduler_manage_port)
cmd_sched += " --server_config_path={}".format(server_config_path)
cmd_sched += " > {}/scheduler/scheduler.log 2>&1 &".format(cur_dir)

print(cmd_sched)
subprocess.call(['bash', '-c', cmd_sched])

for key in args.__dict__:
    print('[',  key, ']',  args.__dict__[key])
