# Copyright 2021-2022 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

import argparse
import subprocess
import os
import ast

cur_dir = os.getcwd()
code_name = 'vae_task'

parser = argparse.ArgumentParser(description="Run {} case".format(code_name))
parser.add_argument("--enable_fl", type=bool, default=True)
parser.add_argument("--device_target", type=str, default="CPU")
parser.add_argument("--server_mode", type=str, default="FEDERATED_LEARNING")
parser.add_argument("--server_num", type=int, default=3)
parser.add_argument("--scheduler_ip", type=str, default="10.113.216.40")
parser.add_argument("--scheduler_port", type=int, default=4444)
parser.add_argument("--fl_server_port", type=int, default=3003)
parser.add_argument("--start_fl_job_threshold", type=int, default=2)
parser.add_argument("--start_fl_job_time_window", type=int, default=30000000)
parser.add_argument("--update_model_ratio", type=float, default=1.0)
parser.add_argument("--update_model_time_window", type=int, default=30000000)
parser.add_argument("--fl_name", type=str, default="VAE")
parser.add_argument("--fl_iteration_num", type=int, default=5)
parser.add_argument("--client_epoch_num", type=int, default=30)
parser.add_argument("--client_batch_size", type=int, default=32)
parser.add_argument("--client_learning_rate", type=float, default=0.001)
# The number of servers that this script will launch.
parser.add_argument("--local_server_num", type=int, default=-1)
parser.add_argument("--encrypt_type", type=str, default="NOT_ENCRYPT",
                    choices=["NOT_ENCRYPT", "DP_ENCRYPT", "SIGNDS", "PW_ENCRYPT"])
parser.add_argument("--config_file_path", type=str, default="{}/config.json".format(cur_dir))
parser.add_argument("--enable_ssl", type=ast.literal_eval, default=False)
# parameters for encrypt_type="DP_ENCRYPT"
parser.add_argument("--dp_eps", type=float, default=200.0)
parser.add_argument("--dp_delta", type=float, default=0.9)  # usually equals 1/start_fl_job_threshold
parser.add_argument("--dp_norm_clip", type=float, default=0.00000001)
# parameters for encrypt_type="SIGNDS"
parser.add_argument("--sign_k", type=float, default=0.01)
parser.add_argument("--sign_eps", type=float, default=100)
parser.add_argument("--sign_thr_ratio", type=float, default=0.6)
parser.add_argument("--sign_global_lr", type=float, default=2)
parser.add_argument("--sign_dim_out", type=int, default=0)
# parameters for encrypt_type="PW_ENCRYPT"
parser.add_argument("--share_secrets_ratio", type=float, default=1.0)
parser.add_argument("--cipher_time_window", type=int, default=3000)
parser.add_argument("--reconstruct_secrets_threshold", type=int, default=2)
parser.add_argument("--root_first_ca_path", type=str, default="")
parser.add_argument("--root_second_ca_path", type=str, default="")
parser.add_argument("--pki_verify", type=ast.literal_eval, default=False)
parser.add_argument("--equip_crl_path", type=str, default="")
# Time difference limit for Anti-Replay Attacks. The default value is 60 seconds.
parser.add_argument("--replay_attack_time_diff", type=int, default=60000)
parser.add_argument("--http_url_prefix", type=str, default="")
parser.add_argument('--global_iteration_time_window', type=int, default=3600000)
# init model path
parser.add_argument("--model_path", type=str, default="none")
# vae args
parser.add_argument("--server_config_path", type=str, default="./Horizontal_FL/conf/server_config.yaml")
parser.add_argument("--ms_node_id", type=str, default="fl-server")

args, _ = parser.parse_known_args()
enable_fl = args.enable_fl
device_target = args.device_target
server_mode = args.server_mode
server_num = args.server_num
scheduler_ip = args.scheduler_ip
scheduler_port = args.scheduler_port
fl_server_port = args.fl_server_port
start_fl_job_threshold = args.start_fl_job_threshold
start_fl_job_time_window = args.start_fl_job_time_window
update_model_ratio = args.update_model_ratio
update_model_time_window = args.update_model_time_window
fl_name = args.fl_name
fl_iteration_num = args.fl_iteration_num
client_epoch_num = args.client_epoch_num
client_batch_size = args.client_batch_size
client_learning_rate = args.client_learning_rate
local_server_num = args.local_server_num
config_file_path = args.config_file_path
enable_ssl = args.enable_ssl
encrypt_type = args.encrypt_type
dp_eps = args.dp_eps
dp_delta = args.dp_delta
dp_norm_clip = args.dp_norm_clip
sign_k = args.sign_k
sign_eps = args.sign_eps
sign_thr_ratio = args.sign_thr_ratio
sign_global_lr = args.sign_global_lr
sign_dim_out = args.sign_dim_out
share_secrets_ratio = args.share_secrets_ratio
cipher_time_window = args.cipher_time_window
reconstruct_secrets_threshold = args.reconstruct_secrets_threshold
root_first_ca_path = args.root_first_ca_path
root_second_ca_path = args.root_second_ca_path
pki_verify = args.pki_verify
equip_crl_path = args.equip_crl_path
replay_attack_time_diff = args.replay_attack_time_diff
model_path = args.model_path
http_url_prefix = args.http_url_prefix
global_iteration_time_window = args.global_iteration_time_window
# vae args
server_config_path = args.server_config_path
ms_node_id = args.ms_node_id

os.environ["MS_NODE_ID"] = ms_node_id
cmd_server = "unset http_proxy && unset https_proxy && "
cmd_server += "rm -rf {}/server_{}/ && ".format(cur_dir, ms_node_id)
cmd_server += "mkdir {}/server_{}/ && ".format(cur_dir, ms_node_id)
cmd_server += "export GLOG_v=1 && "
cmd_server += "python {}/{}.py".format(cur_dir, code_name)
cmd_server += " --enable_fl={}".format(enable_fl)
cmd_server += " --device_target={}".format(device_target)
cmd_server += " --server_mode={}".format(server_mode)
cmd_server += " --ms_role=MS_SERVER"
cmd_server += " --server_num={}".format(server_num)
cmd_server += " --scheduler_ip={}".format(scheduler_ip)
cmd_server += " --scheduler_port={}".format(scheduler_port)
cmd_server += " --fl_server_port={}".format(fl_server_port)
cmd_server += " --start_fl_job_threshold={}".format(start_fl_job_threshold)
cmd_server += " --start_fl_job_time_window={}".format(start_fl_job_time_window)
cmd_server += " --update_model_ratio={}".format(update_model_ratio)
cmd_server += " --update_model_time_window={}".format(update_model_time_window)
cmd_server += " --fl_name={}".format(fl_name)
cmd_server += " --fl_iteration_num={}".format(fl_iteration_num)
cmd_server += " --config_file_path={}".format(config_file_path)
cmd_server += " --enable_ssl={}".format(enable_ssl)
cmd_server += " --client_epoch_num={}".format(client_epoch_num)
cmd_server += " --client_batch_size={}".format(client_batch_size)
cmd_server += " --client_learning_rate={}".format(client_learning_rate)
cmd_server += " --encrypt_type={}".format(encrypt_type)
cmd_server += " --dp_eps={}".format(dp_eps)
cmd_server += " --dp_delta={}".format(dp_delta)
cmd_server += " --dp_norm_clip={}".format(dp_norm_clip)
cmd_server += " --sign_k={}".format(sign_k)
cmd_server += " --sign_eps={}".format(sign_eps)
cmd_server += " --sign_thr_ratio={}".format(sign_thr_ratio)
cmd_server += " --sign_global_lr={}".format(sign_global_lr)
cmd_server += " --sign_dim_out={}".format(sign_dim_out)
cmd_server += " --share_secrets_ratio={}".format(share_secrets_ratio)
cmd_server += " --cipher_time_window={}".format(cipher_time_window)
cmd_server += " --reconstruct_secrets_threshold={}".format(reconstruct_secrets_threshold)
cmd_server += " --root_first_ca_path={}".format(root_first_ca_path)
cmd_server += " --root_second_ca_path={}".format(root_second_ca_path)
cmd_server += " --pki_verify={}".format(pki_verify)
cmd_server += " --equip_crl_path={}".format(equip_crl_path)
cmd_server += " --replay_attack_time_diff={}".format(replay_attack_time_diff)
cmd_server += " --http_url_prefix={}".format(http_url_prefix)
cmd_server += " --model_path={}".format(model_path)
cmd_server += " --global_iteration_time_window={}".format(global_iteration_time_window)
cmd_server += " --server_config_path={}".format(server_config_path)
cmd_server += " > {}/server_{}/server.log 2>&1 &".format(cur_dir, ms_node_id)

print(cmd_server)
subprocess.call(['bash', '-c', cmd_server])
import time
time.sleep(0.3)

for key in args.__dict__:
    print('[',  key, ']',  args.__dict__[key])
